 /*@@
   @file      InitialData.c
   @date      
   @author    Werner Benger   
   @desc 
              Initial data for the 3D Wave Equation
	      Derived from Tom Goodale
   @enddesc 
   @version $Header: /numrelcvs/HawkeCVS/Public/CactusMoL/IDScalarWaveMoLC/src/InitialData.c,v 1.6 2003/02/28 16:09:51 hawke Exp $
 @@*/

#include <math.h>

#include "cctk.h" 
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

/*
#ifndef DEBUG_MOL
#define DEBUG_MOL
#endif
*/

static const char *rcsid = "$Header: /numrelcvs/HawkeCVS/Public/CactusMoL/IDScalarWaveMoLC/src/InitialData.c,v 1.6 2003/02/28 16:09:51 hawke Exp $";

CCTK_FILEVERSION(CactusWave_IDScalarWaveMoLC_InitialData_c)

static CCTK_REAL sqr(CCTK_REAL val)
{
  return val*val;
}

void IDScalarWaveMoLC_InitialData(CCTK_ARGUMENTS);


 /*@@
   @routine    IDScalarWaveMoLC_InitialData
   @date       
   @author     Tom Goodale
   @desc 
               Set up initial data for the wave equation
   @enddesc 
   @calls      
   @calledby   
   @history 
   @hdate Mon Oct 11 11:48:03 1999 @hauthor Werner Benger
   @hdesc  Converted to C++
   @hdate Mon Oct 11 11:48:20 1999 @hauthor Tom Goodale
   @hdesc Added the rest of the initial data.
   @hdate Thu Feb 17 09:22:01 2000 @hauthor Tom Goodale
   @hdesc Converted to C
   @endhistory 

@@*/

void IDScalarWaveMoLC_InitialData(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  
  int i,j,k;

  CCTK_REAL dt;
  CCTK_REAL omega;
  int index;
  CCTK_REAL X, Y, Z, R;
  CCTK_REAL pi;

#ifdef DEBUG_MOL
  printf("Initial data says sigma = %i, radius = %i\n", sigma, radius);
#endif

  dt = CCTK_DELTA_TIME;

  if(CCTK_Equals(initial_data, "plane"))
  {
    omega = sqrt(sqr(kx)+sqr(ky)+sqr(kz));

    for(k=0; k<cctk_lsh[2]; k++)
    {
      for(j=0; j<cctk_lsh[1]; j++)
      {
        for(i=0; i<cctk_lsh[0]; i++)
        {
          index =  CCTK_GFINDEX3D(cctkGH,i,j,k);

          phi[index]     = amplitude*
            cos(kx*x[index]+ky*y[index]+kz*z[index]+omega*cctk_time);
          phit[index] = -omega*amplitude*
            sin(kx*x[index]+ky*y[index]+kz*z[index]+omega*cctk_time);
          phix[index] = -kx*amplitude*
            sin(kx*x[index]+ky*y[index]+kz*z[index]+omega*cctk_time);
          phiy[index] = -ky*amplitude*
            sin(kx*x[index]+ky*y[index]+kz*z[index]+omega*cctk_time);
          phiz[index] = -kz*amplitude*
            sin(kx*x[index]+ky*y[index]+kz*z[index]+omega*cctk_time);
        }
      }
    }
  }
  else if(CCTK_Equals(initial_data, "gaussian"))
  {
    for(k=0; k<cctk_lsh[2]; k++)
    {
      for(j=0; j<cctk_lsh[1]; j++)
      {
        for(i=0; i<cctk_lsh[0]; i++)
        {
          index =  CCTK_GFINDEX3D(cctkGH,i,j,k);

          X = x[index] - centrex;
          Y = y[index] - centrey;
          Z = z[index] - centrez;

          R = sqrt(X*X + Y*Y + Z*Z);

          phi[index] = amplitude*exp( - sqr( (R - radius) / sigma ) );
          phit[index] = 0.0;
          if (R > 1.e-10) {
            phix[index] = -2.0*(R-radius)/sigma/sigma*X/R*phi[index];
            phiy[index] = -2.0*(R-radius)/sigma/sigma*Y/R*phi[index];
            phiz[index] = -2.0*(R-radius)/sigma/sigma*Z/R*phi[index];
          } 
          else {
            phix[index] = 0.0;
            phiy[index] = 0.0;
            phiz[index] = 0.0;
          } 
        }
      }
    }
  }
  else if(CCTK_Equals(initial_data, "box"))
  {
    pi = 4.0*atan(1.0);
    omega = sqrt(sqr(kx)+sqr(ky)+sqr(kz));

    for(k=0; k<cctk_lsh[2]; k++)
    {
      for(j=0; j<cctk_lsh[1]; j++)
      {
        for(i=0; i<cctk_lsh[0]; i++)
        {
          index =  CCTK_GFINDEX3D(cctkGH,i,j,k);

          phi[index] = amplitude*sin(kx*(x[index]-0.5)*pi)*
                                 sin(ky*(y[index]-0.5)*pi)*
                                 sin(kz*(z[index]-0.5)*pi)*
                                 cos(omega*cctk_time*pi);
          phit[index] = -omega*pi*amplitude*sin(kx*(x[index]-0.5)*pi)*
                                 sin(ky*(y[index]-0.5)*pi)*
                                 sin(kz*(z[index]-0.5)*pi)*
                                 sin(omega*cctk_time*pi);
          phix[index] = kx*pi*amplitude*cos(kx*(x[index]-0.5)*pi)*
                                 sin(ky*(y[index]-0.5)*pi)*
                                 sin(kz*(z[index]-0.5)*pi)*
                                 cos(omega*cctk_time*pi);
          phiy[index] = kx*pi*amplitude*sin(kx*(x[index]-0.5)*pi)*
                                 cos(ky*(y[index]-0.5)*pi)*
                                 sin(kz*(z[index]-0.5)*pi)*
                                 cos(omega*cctk_time*pi);
          phiz[index] = kx*pi*amplitude*sin(kx*(x[index]-0.5)*pi)*
                                 sin(ky*(y[index]-0.5)*pi)*
                                 cos(kz*(z[index]-0.5)*pi)*
                                 cos(omega*cctk_time*pi);
        }
      }
    }
  }

  /* For the dependency test */

  if (test_dependency) {

    for(k=0; k<cctk_lsh[2]; k++)
    {
      for(j=0; j<cctk_lsh[1]; j++)
      {
        for(i=0; i<cctk_lsh[0]; i++)
        {
          index =  CCTK_GFINDEX3D(cctkGH,i,j,k);
          
          junk[index] = sin(x[index]);
        }
      }
    }
    
  }
  
        

  return;
}

