#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"
#include "cctk_DefineThorn.h"
#include "util_Table.h"

/*
   -> routine    Schroedinger_Boundaries
   -> date       April 2005
   -> author     Francisco S. Guzman
*/
               
      subroutine Schroedinger_Boundaries(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

!     Local variables

      INTEGER    i, j, k, nx, ny, nz
      INTEGER    cctk_equals

      CCTK_REAL  dx, dy, dz, dt, dtw
      CCTK_REAL  idx, idy, idz
      CCTK_REAL  dx2i, dy2i, dz2i, dt2

      CCTK_REAL  zero, one, two, half, third
      CCTK_REAL  rdot, idot

      integer :: ierr = -1
      integer,dimension(3):: sw     = 1
      CCTK_REAL,parameter :: sval   = 0.0
      CCTK_REAL,parameter :: rzero  = 0.0
      CCTK_REAL,parameter :: rone   = 1.0
      integer,parameter   :: swdir  = 1
      integer,parameter   :: ione   = 1,   mione   = -1
      integer,parameter   :: itwo   = 2,   mitwo   = -2
      integer,parameter   :: ithree = 3,   mithree = -3
      integer,parameter   :: ifour  = 4,   izero   =  0

!  Useful numbers

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      zero  = 0.0D0
      one   = 1.0D0
      two   = 2.0D0
      half  = 0.5D0

      dx = CCTK_DELTA_SPACE(1)
      dy = CCTK_DELTA_SPACE(2)
      dz = CCTK_DELTA_SPACE(3)
      dt = CCTK_DELTA_TIME

      idx = one/dx 
      idy = one/dy 
      idz = one/dz

      call CartSymGN(ierr,cctkGH,"schroedinger::Sch_wavefunction")

      if (ierr < 0)  then
        call CCTK_WARN(0,"Schroedinger_Boundaries: Error in boundary routines - giving up!")      
      end if
      
      end subroutine Schroedinger_Boundaries  

