 /*@@
   @file      WaveToyMoL.c
   @date      Fri Nov  9 13:33:25 2001
   @author    Ian Hawke
   @desc 
   The equivalent of WaveToy.c
   @enddesc 
 @@*/

  
#include "cctk.h" 
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

#include "Boundary.h"
#include "Symmetry.h"

/*
#ifndef DEBUG_MOL
#define DEBUG_MOL
#endif
*/

void WaveToyMoL_Boundaries(CCTK_ARGUMENTS);
void WaveToyMoL_CalcRHS(CCTK_ARGUMENTS);

 /*@@
   @routine    WaveToyMoL_Boundaries
   @date       Fri Nov  9 13:34:47 2001
   @author     Ian Hawke
   @desc 
   A slight alteration of the standard routine. Note that I 
   haven't set up the error checking correctly as yet.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

void WaveToyMoL_Boundaries(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  int ierr=-1;
  int sw[3];

#ifdef DEBUG_MOL
  printf("We've reached the Boundary enforcement; step must be done!\n");
#endif

  /* Set the stencil width */
  sw[0]=1;
  sw[1]=1;
  sw[2]=1;

  /* Uses all default arguments, so invalid table handle -1 can be passed */
  ierr = Boundary_SelectGroupForBC
    (cctkGH, CCTK_ALL_FACES, 1, -1, "wavetoymol::scalarevolvemol", bound);
  ierr = Boundary_SelectGroupForBC
    (cctkGH, CCTK_ALL_FACES, 1, -1, "wavetoymol::scalarevolvemol_spatial", bound);

  if (ierr < 0) 
  {
    CCTK_WARN(0,"Boundary conditions not applied - giving up!");
  }

  return;
}

 /*@@
   @routine    WaveToyMoL_CalcRHS
   @date       Fri Nov  9 13:37:27 2001
   @author     Ian Hawke
   @desc 
   The routine that calculates the RHS of the PDE for the MoL step.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

void  WaveToyMoL_CalcRHS(CCTK_ARGUMENTS)
{

  DECLARE_CCTK_ARGUMENTS
      
  int i,j,k;
  int index;
  int istart, jstart, kstart, iend, jend, kend;
  CCTK_REAL dx,dy,dz;
  CCTK_REAL hdxi, hdyi, hdzi;

#ifdef DEBUG_MOL
  printf("In routine to calculate RHS for wavetoy\n");
#endif

  /* Set up shorthands */
  dx = CCTK_DELTA_SPACE(0);
  dy = CCTK_DELTA_SPACE(1);
  dz = CCTK_DELTA_SPACE(2);

  hdxi = 0.5 / dx;
  hdyi = 0.5 / dy;
  hdzi = 0.5 / dz;
  
  istart = 1;
  jstart = 1;
  kstart = 1;
  
  iend = cctk_lsh[0]-1;
  jend = cctk_lsh[1]-1;
  kend = cctk_lsh[2]-1;

  /* Calculate the right hand sides. */

#ifdef DEBUG_MOL
  printf("About to loop.\n");
#endif

  for (k=0; k<cctk_lsh[2]; k++)
  {
    for (j=0; j<cctk_lsh[1]; j++)
    {
      for (i=0; i<cctk_lsh[0]; i++)
      {
        index = CCTK_GFINDEX3D(cctkGH,i,j,k);
        phirhs[index] = 0;
        phitrhs[index] = 0;
        phixrhs[index] = 0;
        phiyrhs[index] = 0;
        phizrhs[index] = 0;
      }
    }
  }  

  for (k=kstart; k<kend; k++)
  {
    for (j=jstart; j<jend; j++)
    {
      for (i=istart; i<iend; i++)
      {
        index = CCTK_GFINDEX3D(cctkGH,i,j,k);
        
        phirhs[index] = phit[index];
        phitrhs[index] = 
            (phix[CCTK_GFINDEX3D(cctkGH, i+1, j, k)] -
             phix[CCTK_GFINDEX3D(cctkGH, i-1, j, k)]) *hdxi
          + (phiy[CCTK_GFINDEX3D(cctkGH, i, j+1, k)] -
             phiy[CCTK_GFINDEX3D(cctkGH, i, j-1, k)]) *hdyi
          + (phiz[CCTK_GFINDEX3D(cctkGH, i, j, k+1)] -
             phiz[CCTK_GFINDEX3D(cctkGH, i, j, k-1)]) *hdzi;
        phixrhs[index] = (phit[CCTK_GFINDEX3D(cctkGH, i+1, j, k)] -
                          phit[CCTK_GFINDEX3D(cctkGH, i-1, j, k)]) *hdxi;
        phiyrhs[index] = (phit[CCTK_GFINDEX3D(cctkGH, i, j+1, k)] -
                          phit[CCTK_GFINDEX3D(cctkGH, i, j-1, k)]) *hdyi;
        phizrhs[index] = (phit[CCTK_GFINDEX3D(cctkGH, i, j, k+1)] -
                          phit[CCTK_GFINDEX3D(cctkGH, i, j, k-1)]) *hdzi;

#ifdef DEBUG_MOL
        printf("ijk %i %i %i, phitrhs %f\n",i,j,k,phitrhs[index]);
#endif  
      } 
    }
  }

#ifdef DEBUG_MOL
  printf("Done with loop\n");
#endif

  return;
}
