#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"

/*    
   -> routine    RealSF_InitialData
   -> date       January 2002 
   -> author     Francisco S. Guzman
*/


      subroutine RealSF_InitialData(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS
      DECLARE_CCTK_FUNCTIONS

      integer :: i, j, k, nx, ny, nz

      CCTK_REAL  dx, dy, dz, dt, omega, radius
      CCTK_REAL  zero, one, two, half, third, rp, pi

!  Useful numbers

      zero  = 0.0D0
      one   = 1.0D0
      two   = 2.0D0
      half  = 0.5D0
      third = 1.0D0/3.0D0

      dt = cctk_delta_time
      dx = cctk_delta_space(1)
      dy = cctk_delta_space(2)
      dz = cctk_delta_space(3)

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      pi = 4.0*atan(1.0)
         
      omega  = sqrt(kx**2+ky**2+kz**2)
      radius = zero
                  
      if (CCTK_EQUALS(initial_data,"plane")) then
                  
         do k = 1, nz
            do j = 1, ny
               do i = 1, nx

                  rphi(i,j,k) = amplitude*
     .                         cos(kx*x(i,j,k)+ky*y(i,j,k)+kz*z(i,j,k)+omega*cctk_time)
                  rpi(i,j,k) = -omega*amplitude*
     .                          sin(kx*x(i,j,k)+ky*y(i,j,k)+kz*z(i,j,k)+omega*cctk_time)
                  rpsix(i,j,k) = -kx*amplitude*
     .                          sin(kx*x(i,j,k)+ky*y(i,j,k)+kz*z(i,j,k)+omega*cctk_time)
                  rpsiy(i,j,k) = -ky*amplitude*
     .                          sin(kx*x(i,j,k)+ky*y(i,j,k)+kz*z(i,j,k)+omega*cctk_time)
                  rpsiz(i,j,k) = -kz*amplitude*
     .                          sin(kx*x(i,j,k)+ky*y(i,j,k)+kz*z(i,j,k)+omega*cctk_time)
               end do
            end do
         end do

      else if (CCTK_EQUALS(initial_data,"gaussian")) then

         do k = 1, nz
            do j = 1, ny
               do i = 1, nx

! Data for a centered time-symmetric gaussian

                    rphi(i,j,k)  = amplitude*exp(-((x(i,j,k) - x0)**2 + (y(i,j,k) - y0)**2 + (z(i,j,k) - z0)**2)/sigma**2)
     .                           + amplitude*exp(-((x(i,j,k) + x0)**2 + (y(i,j,k) + y0)**2 + (z(i,j,k) + z0)**2)/sigma**2)
                    rpsix(i,j,k) = - two*(x(i,j,k) - x0)*(amplitude*exp(-((x(i,j,k) - x0)**2 + (y(i,j,k) - y0)**2 +
     .                                                                    (z(i,j,k) - z0)**2)/sigma**2))/sigma**2
     .                             - two*(x(i,j,k) + x0)*(amplitude*exp(-((x(i,j,k) + x0)**2 + (y(i,j,k) + y0)**2 +
     .                                                                    (z(i,j,k) + z0)**2)/sigma**2))/sigma**2
                    rpsiy(i,j,k) = - two*(y(i,j,k) - y0)*(amplitude*exp(-((x(i,j,k) - x0)**2 + (y(i,j,k) - y0)**2 +
     .                                                                    (z(i,j,k) - z0)**2)/sigma**2))/sigma**2
     .                             - two*(y(i,j,k) + y0)*(amplitude*exp(-((x(i,j,k) + x0)**2 + (y(i,j,k) + y0)**2 +
     .                                                                    (z(i,j,k) + z0)**2)/sigma**2))/sigma**2
                    rpsiz(i,j,k) = - two*(z(i,j,k) - z0)*(amplitude*exp(-((x(i,j,k) - x0)**2 + (y(i,j,k) - y0)**2 +
     .                                                                    (z(i,j,k) - z0)**2)/sigma**2))/sigma**2
     .                             - two*(z(i,j,k) + z0)*(amplitude*exp(-((x(i,j,k) + x0)**2 + (y(i,j,k) + y0)**2 +
     .                                                                    (z(i,j,k) + z0)**2)/sigma**2))/sigma**2
                    rpi(i,j,k)   = zero

               end do
            end do
         end do

      else if (CCTK_EQUALS(initial_data,"box")) then


         do k = 1, nz
            do j = 1, ny
               do i = 1, nx

                  rphi(i,j,k) = amplitude*sin(kx*(x(i,j,k)-0.5)*pi)*
     .                            sin(ky*(y(i,j,k)-0.5)*pi)*
     .                            sin(kz*(z(i,j,k)-0.5)*pi)*
     .                            cos(omega*cctk_time*pi)
                  rpi(i,j,k) = -omega*pi*amplitude*sin(kx*(x(i,j,k)-0.5)*pi)*
     .                            sin(ky*(y(i,j,k)-0.5)*pi)*
     .                            sin(kz*(z(i,j,k)-0.5)*pi)*
     .                            sin(omega*cctk_time*pi)
                  rpsix(i,j,k) = kx*pi*amplitude*cos(kx*(x(i,j,k)-0.5)*pi)*
     .                            sin(ky*(y(i,j,k)-0.5)*pi)*
     .                            sin(kz*(z(i,j,k)-0.5)*pi)*
     .                            cos(omega*cctk_time*pi)
                  rpsiy(i,j,k) = kx*pi*amplitude*sin(kx*(x(i,j,k)-0.5)*pi)*
     .                            cos(ky*(y(i,j,k)-0.5)*pi)*
     .                            sin(kz*(z(i,j,k)-0.5)*pi)*
     .                            cos(omega*cctk_time*pi)
                  rpsiz(i,j,k) = kx*pi*amplitude*sin(kx*(x(i,j,k)-0.5)*pi)*
     .                            sin(ky*(y(i,j,k)-0.5)*pi)*
     .                            cos(kz*(z(i,j,k)-0.5)*pi)*
     .                            cos(omega*cctk_time*pi)
               end do
            end do
         end do

      end if


      end subroutine RealSF_InitialData



