#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

/*
   -> routine    Schroedinger_Sources
   -> date       April 2005
   -> author     Francisco S. Guzman
   -> comments   Sources for the evolution of the wave function.
*/


      subroutine Schroedinger_Sources(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      INTEGER  i, j, k, nx, ny, nz

      CCTK_REAL  dx, dy, dz, dt, dtw
      CCTK_REAL  idx,idy,idz
      CCTK_REAL  zero, one, two, half, third

      CCTK_REAL  dtfac
      CCTK_REAL  lap1, lap2
      INTEGER    cctk_equals

!  Useful numbers

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      zero  = 0.0D0
      one   = 1.0D0
      two   = 2.0D0
      half  = 0.5D0

      dx = CCTK_DELTA_SPACE(1)
      dy = CCTK_DELTA_SPACE(2)
      dz = CCTK_DELTA_SPACE(3)
      dt = CCTK_DELTA_TIME

      idx = one/dx
      idy = one/dy
      idz = one/dz


! Calling the potential 

      call Schroedinger_Diagnostics(CCTK_ARGUMENTS)
      call Schroedinger_Potential(CCTK_ARGUMENTS)

        do k = 2, nz - 1
           do j = 2, ny - 1  
              do i = 2, nx - 1

                lap1 = (psi1(i+1,j,k) - two*psi1(i,j,k) + psi1(i-1,j,k))*idx**2 +
     .                 (psi1(i,j+1,k) - two*psi1(i,j,k) + psi1(i,j-1,k))*idy**2 +
     .                 (psi1(i,j,k+1) - two*psi1(i,j,k) + psi1(i,j,k-1))*idz**2

                lap2 = (psi2(i+1,j,k) - two*psi2(i,j,k) + psi2(i-1,j,k))*idx**2 +
     .                 (psi2(i,j+1,k) - two*psi2(i,j,k) + psi2(i,j-1,k))*idy**2 +
     .                 (psi2(i,j,k+1) - two*psi2(i,j,k) + psi2(i,j,k-1))*idz**2

                spsi1(i,j,k) = -half*lap2 + iV(i,j,k)*psi1(i,j,k) + 
     .                         rV(i,j,k)*psi2(i,j,k) + CC*rho(i,j,k)*psi2(i,j,k)

                spsi2(i,j,k) =  half*lap1 + iV(i,j,k)*psi2(i,j,k) - 
     .                         rV(i,j,k)*psi1(i,j,k) - CC*rho(i,j,k)*psi1(i,j,k)

              end do  
           end do
        end do

      end subroutine Schroedinger_Sources


